﻿using ReadWriteCsv;
using System;
using System.Windows.Forms;
using System.Text;
using System.Collections.Generic;
using System.IO;
using System.Drawing;
using System.Reflection;

namespace GoodReadsApp
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        #region FormLevelVariables

        string FilePathAndName = string.Empty;
        string appTitle = "Goodreads CSV Reader";
        int titleIndex = 1;
        int authorIndex = 3;
        int additionalAuthorsIndex = 4;
        int myRatingIndex = 7;
        int communityRatingIndex = 8;
        int publisherIndex = 9;
        int bindingIndex = 10;
        int numPagesIndex = 11;
        int pubDateIndex = 12;
        int origPubYearIndex = 13;
        int dateReadIndex = 14;
        int dateAddedIndex = 15;
        int bookshelfIndex = 18;
        int reviewIndex = 19;        

        int totalRows = 0;
        int totalUniqueAuthors = 0;
        int booksWithAdditionalAuthors = 0;
        int totPagesAcrossBooks = 0;
        int alreadyRead;
        int toBeRead;
        int currentlyReading;

        long bytesInOpenFile = 0;
        decimal kbInOpenFile = 0.0M;

        string descFileSize = string.Empty;
        string filenameOfOpenedCSVfile = string.Empty;
        string itemClicked = string.Empty;

        StringBuilder sbBooksAlreadyRead = new StringBuilder();
        StringBuilder sbBooksToBeRead = new StringBuilder();
        StringBuilder sbBooksCurrentlyReading = new StringBuilder();
        StringBuilder sbOther = new StringBuilder();

        Form fAbout = new AboutBox1();

        #endregion

        private void Form1_Load(object sender, EventArgs e)
        {
            this.Text = appTitle;
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;            
            pictureBox1.Image = Properties.Resources._2013_04_02_155739;
            pictureBox1.BackgroundImageLayout = ImageLayout.None;
            lblTitles.Text = "Book titles in alphabetical order";
            lblTitles.Enabled = false;
            statusStrip1.BackColor = Color.White;
            toolStripStatusLabel1.Text = string.Empty;            
            lblFindByAuthor.Enabled = false;
            lstAuthors.Enabled = false;
            btnRefresh.Visible = false;
            initListView();
        }      

        private int getTotalRowsInCsvFile()
        {
            using (CsvFileReader reader = new CsvFileReader(FilePathAndName))
            {
                int counter = 0;
                CsvRow row = new CsvRow();
                while (reader.ReadRow(row))
                {
                    counter++;
                }
                return counter;
            }
        }       

        private void openToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "CSV file (*.csv)|*.csv";
            ofd.RestoreDirectory = false;
            ofd.Title = "Browse to Good Reads .csv file...";
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                this.Text = appTitle + " (reading data from CSV file...)";
                FilePathAndName = ofd.FileName;
                filenameOfOpenedCSVfile = Path.GetFileName(FilePathAndName);
                FileInfo fi = new FileInfo(FilePathAndName);               
                bytesInOpenFile = fi.Length;
                descFileSize = string.Empty;
                if (bytesInOpenFile <= 1023)
                {
                    descFileSize = bytesInOpenFile.ToString() + " bytes";
                }
                else
                {
                    kbInOpenFile = (decimal)bytesInOpenFile / 1024;                    
                    descFileSize = Math.Round(kbInOpenFile, 2).ToString() + " KB";
                }

                using (CsvFileReader reader = new CsvFileReader(FilePathAndName))
                {
                    alreadyRead = 0;
                    toBeRead = 0;
                    currentlyReading = 0;

                    CsvRow row = new CsvRow();
                    totalRows = getTotalRowsInCsvFile();                                                         
                    int counter = 0;
                    while (reader.ReadRow(row) && counter <= totalRows)
                    {
                        counter++;
                        string[] values = row.ToArray();

                        if (counter > 1)
                        {
                            try
                            {
                                addBookInfo(values[bookshelfIndex], values, counter);
                            }
                            catch (Exception ex)
                            {
                                string moreInfo = string.Empty;
                                if (ex.Message.Contains("Index")) { moreInfo += " You probably tried to load a non-GoodReads CSV file..."; }
                                MessageBox.Show(ex.Message + moreInfo, "Exception");
                                return;                             
                            }
                        }
                    }

                    appendBookInfoToTextBox();           
                    
                    totalRows -= 1; //so that we don't counter the initial line of header info as a book record...
                    
                    lblFindByAuthor.Enabled = true;                 
                    lstAuthors.Enabled = true;
                    lblTitles.Enabled = true;
                    updateToolStripStatusBarLabelAfterLoadingFile();
                    if (totalUniqueAuthors == lstAuthors.Items.Count)
                    {
                        lblFindByAuthor.Text = "Find books by author name:" + Environment.NewLine + "(" + totalUniqueAuthors.ToString() + " authors found)";
                        //MessageBox.Show(lstAuthors.Items.Count.ToString());
                    }

                    //next closing brace ends using (CsvFileReader...

                    //MessageBox.Show("Total # books that have more than one author: " + booksWithAdditionalAuthors.ToString());
                }
            }
        }      

        private void initListView()
        {
            listView1.GridLines = true;
            listView1.View = View.Details;
            listView1.Scrollable = true;
            listView1.Font = new Font("Tahoma", 9);
            

            listView1.Columns.Add(new ColumnHeader());
            listView1.Columns[0].Text = "Title";
            listView1.Columns[0].Width = 250;

            listView1.Columns.Add(new ColumnHeader());
            listView1.Columns[1].Text = "Author";
            listView1.Columns[1].Width = 165;

            listView1.Columns.Add(new ColumnHeader());
            listView1.Columns[2].Text = "Publisher";
            listView1.Columns[2].Width = 180;

            listView1.Columns.Add(new ColumnHeader());
            listView1.Columns[3].Text = "Original Publication Date";
            listView1.Columns[3].Width = 170;
            listView1.Columns[3].TextAlign = HorizontalAlignment.Center;

            listView1.Columns.Add(new ColumnHeader());
            listView1.Columns[4].Text = "Pages";
            listView1.Columns[4].Width = 100;

            listView1.Columns.Add(new ColumnHeader());
            listView1.Columns[5].Text = "Binding";
            listView1.Columns[5].Width = 150;

            listView1.Columns.Add(new ColumnHeader());
            listView1.Columns[6].Text = "Current Bookshelf";
            listView1.Columns[6].Width = 125;

            listView1.Columns.Add(new ColumnHeader());
            listView1.Columns[7].Text = "Community Rating";
            listView1.Columns[7].Width = 150;

            listView1.Columns.Add(new ColumnHeader());
            listView1.Columns[8].Text = "My Rating";
            listView1.Columns[8].Width = 125;

            listView1.Show();

            
        }

        private void appendBookInfoToTextBox()
        {
            if (sbBooksCurrentlyReading != null)
            {
                textBox1.AppendText(sbBooksCurrentlyReading.ToString());
            }

            if (sbBooksToBeRead != null)
            {
                textBox1.AppendText(sbBooksToBeRead.ToString());
            }

            if (sbBooksAlreadyRead != null)
            {
                textBox1.AppendText(sbBooksAlreadyRead.ToString());
            }

            if (sbOther != null)
            {
                textBox1.AppendText(sbOther.ToString());
            }

            scrollToTopOfTextBox(textBox1);

            this.Text = appTitle + " (loaded file \"" +filenameOfOpenedCSVfile + "\" is " + descFileSize + ")";
            
        }

        private void updateToolStripStatusBarLabelAfterLoadingFile()
        {
            toolStripStatusLabel1.Text = totalRows.ToString() + " books (totaling ";
            toolStripStatusLabel1.Text += String.Format("{0:#,##0}", totPagesAcrossBooks) + " pages); ";           
            toolStripStatusLabel1.Text += "Currently reading: " + currentlyReading.ToString() + "; ";
            toolStripStatusLabel1.Text += "To be read: " + toBeRead.ToString() + "; ";
            toolStripStatusLabel1.Text += "Already read: " + alreadyRead.ToString();
            if (totalRows > (currentlyReading + toBeRead + alreadyRead))
            {
                toolStripStatusLabel1.Text += "; Other: " + (totalRows - (currentlyReading + toBeRead + alreadyRead)).ToString();
            }
        }

        private void scrollToTopOfTextBox(TextBox tb)
        {
            tb.SelectionStart = 0;
            tb.SelectionLength = 0;
            tb.ScrollToCaret();
        }

        private void addBookInfo(string _readCategory, string[] _values, int _counter)
        {            

            StringBuilder sbTemp = new StringBuilder();
            sbTemp.Clear();
            sbTemp.Append("\"" + _values[titleIndex] + "\"" + Environment.NewLine);            

            #region buildUpThisTempStringBuilderInstanceWithMoreData

            sbTemp.Append("Author: " + _values[authorIndex] + Environment.NewLine);
            if (!lstAuthors.Items.Contains(_values[authorIndex]))
            {
                lstAuthors.Items.Add(_values[authorIndex]);
                totalUniqueAuthors++;
            }

            if (_values[additionalAuthorsIndex] != string.Empty)
            {
                sbTemp.Append("Additional author(s): " + _values[additionalAuthorsIndex] + Environment.NewLine);
                booksWithAdditionalAuthors++;
            }

            if (_values[pubDateIndex] == string.Empty) { _values[pubDateIndex] = "pub. date?"; }
            sbTemp.Append("Publisher: " + _values[publisherIndex] + " (" + _values[pubDateIndex] + ")" + Environment.NewLine);
            
            if (_values[origPubYearIndex] == string.Empty) { _values[origPubYearIndex] = "?"; }
            sbTemp.Append("Original Publication Year: " + _values[origPubYearIndex] + Environment.NewLine);

            if (_values[numPagesIndex] == string.Empty || _values[numPagesIndex] == "0") { 
                _values[numPagesIndex] = "?"; 
            }
            sbTemp.Append("Pages: " + _values[numPagesIndex] + Environment.NewLine);
            int pagesToAdd = 0;
            bool result = int.TryParse(_values[numPagesIndex], out pagesToAdd);
            if (result)
            {
                totPagesAcrossBooks += pagesToAdd;
            }

            if (_values[bindingIndex] == string.Empty) { _values[bindingIndex] = "?"; }
            sbTemp.Append("Binding: " + _values[bindingIndex] + Environment.NewLine);

            if (_values[dateAddedIndex] == string.Empty) { _values[dateAddedIndex] = "?"; }
            sbTemp.Append("Date Added: " + _values[dateAddedIndex] + Environment.NewLine);

            if (_values[dateReadIndex] == string.Empty) { _values[dateReadIndex] = "?"; }
            sbTemp.Append("Date Read: " + _values[dateReadIndex] + Environment.NewLine);

            if (_values[bookshelfIndex] == string.Empty) { _values[bookshelfIndex] = "?"; }
            sbTemp.Append("Bookshelf: " + _values[bookshelfIndex] + Environment.NewLine);

            if (_values[communityRatingIndex] != string.Empty)
            {
                sbTemp.Append("GoodReads Community Rating: " + _values[communityRatingIndex] + Environment.NewLine);
            }

            if (_values[myRatingIndex] != string.Empty && _values[myRatingIndex] != "0")
            {
                sbTemp.Append("My Rating: " + _values[myRatingIndex] + Environment.NewLine);
            }

            if (_readCategory == "read")
            {
                if (_values[myRatingIndex] != string.Empty)
                {
                    sbTemp.Append("My Rating: " + _values[myRatingIndex] + " stars" + Environment.NewLine);
                }
            }

            sbTemp.Append(Environment.NewLine + "My review of \"");
            sbTemp.Append(_values[titleIndex] + "\": ");
            if (_values[reviewIndex] != string.Empty)
            {
                sbTemp.Append(_values[reviewIndex] + Environment.NewLine);
            }
            else
            {
                sbTemp.Append("No review posted." + Environment.NewLine);
            }

            #endregion

            if (_counter < totalRows)
            {
                sbTemp.Append(Environment.NewLine + Environment.NewLine);
                sbTemp.Append("--------------- End of Book Record ---------------" + Environment.NewLine + Environment.NewLine + Environment.NewLine + Environment.NewLine);
            }

            #region appendToOneofThreeFormLevelStringBuilderInstancesBasedUponReadCategory
            if (_readCategory == "read")
            {
                sbBooksAlreadyRead.Append(sbTemp.ToString());
                alreadyRead++;
            }
            else
            {
                if (_readCategory == "to-read")
                {
                    sbBooksToBeRead.Append(sbTemp.ToString());
                    toBeRead++;
                }
                else
                {
                    if (_readCategory == "currently-reading")
                    {
                        sbBooksCurrentlyReading.Append(sbTemp.ToString());
                        currentlyReading++;
                    }
                    else
                    {
                        sbOther.Append(sbTemp.ToString());
                    }
                }
            }
            #endregion

            string[] fields = new string[9];
            fields[0] = _values[titleIndex];
            fields[1] = _values[authorIndex];
            fields[2] = _values[publisherIndex];
            fields[3] = _values[pubDateIndex];
            fields[4] = _values[numPagesIndex];
            fields[5] = _values[bindingIndex];
            fields[6] = _values[bookshelfIndex];
            fields[7] = _values[communityRatingIndex] + " stars";

            if (_values[myRatingIndex] == "0")
            {
                fields[8] = "I haven't yet rated it";
            }
            else
            {
                fields[8] = _values[myRatingIndex] + " stars";
            }
           
            listView1.Sorting = SortOrder.Ascending;
            ListViewItem lvi = new ListViewItem(fields);
            listView1.Items.Add(lvi);            

        }

        private void exitToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private StringBuilder getEntireRowBySpecificTitle(string specific_title)
        {
            StringBuilder sbTemp = new StringBuilder();

            using (CsvFileReader reader = new CsvFileReader(FilePathAndName))
            {
                CsvRow row = new CsvRow();

                int counter = 0;
                while (reader.ReadRow(row) && counter <= totalRows)
                {
                    counter++;
                    string[] values = row.ToArray();
                    if (values[titleIndex] == specific_title)
                    {
                        sbTemp.Append("Title: " + values[titleIndex] + Environment.NewLine + Environment.NewLine);
                        sbTemp.Append("Author: " + values[authorIndex] + Environment.NewLine + Environment.NewLine);
                        sbTemp.Append("Publisher: " + values[publisherIndex] + Environment.NewLine + Environment.NewLine);
                        sbTemp.Append("Date published: " + values[pubDateIndex] + Environment.NewLine + Environment.NewLine);
                        sbTemp.Append("Pages: " + values[numPagesIndex] + Environment.NewLine + Environment.NewLine);

                        if (values[bookshelfIndex] == "read")
                        {
                            if (values[dateReadIndex] == string.Empty)
                            {
                                values[dateReadIndex] = "finished book on date ???";
                            }
                        }

                        if (values[bookshelfIndex] == "currently-reading")
                        {
                            values[dateReadIndex] = "still reading...";
                        }

                        if (values[bookshelfIndex] == "to-read")
                        {
                            values[dateReadIndex] = "not yet read";
                        }
                        
                        sbTemp.Append("Date read: " + values[dateReadIndex] + Environment.NewLine + Environment.NewLine);
                        break;
                    }
                }
            }

            return sbTemp;
        }

        private StringBuilder getAllRowsBySpecificAuthor(string specific_author, string[] _values)
        {

            StringBuilder sbTemp = new StringBuilder();
            sbTemp.Clear();

            if (_values[authorIndex] == specific_author)
            {

                sbTemp.Append("\"" + _values[titleIndex] + "\"" + Environment.NewLine);

                sbTemp.Append("Author: " + _values[authorIndex] + Environment.NewLine);
                if (!lstAuthors.Items.Contains(_values[authorIndex]))
                {
                    lstAuthors.Items.Add(_values[authorIndex]);
                }

                if (_values[pubDateIndex] == string.Empty) { _values[pubDateIndex] = "pub. date?"; }
                sbTemp.Append("Publisher: " + _values[publisherIndex] + " (" + _values[pubDateIndex] + ")" + Environment.NewLine);

                if (_values[origPubYearIndex] == string.Empty) { _values[origPubYearIndex] = "?"; }
                sbTemp.Append("Original Publication Year: " + _values[origPubYearIndex] + Environment.NewLine);

                if (_values[numPagesIndex] == string.Empty || _values[numPagesIndex] == "0") { _values[numPagesIndex] = "?"; }
                sbTemp.Append("Pages: " + _values[numPagesIndex] + Environment.NewLine);

                if (_values[bindingIndex] == string.Empty) { _values[bindingIndex] = "?"; }
                sbTemp.Append("Binding: " + _values[bindingIndex] + Environment.NewLine);

                if (_values[dateAddedIndex] == string.Empty) { _values[dateAddedIndex] = "?"; }
                sbTemp.Append("Date Added: " + _values[dateAddedIndex] + Environment.NewLine);

                if (_values[dateReadIndex] == string.Empty) 
                { 
                    _values[dateReadIndex] = "?"; 
                }
                sbTemp.Append("Date Read: " + _values[dateReadIndex] + Environment.NewLine);
            

                if (_values[bookshelfIndex] == string.Empty) { _values[bookshelfIndex] = "?"; }
                sbTemp.Append("Bookshelf: " + _values[bookshelfIndex] + Environment.NewLine);


                sbTemp.Append(Environment.NewLine + "My review of \"");
                sbTemp.Append(_values[titleIndex] + "\": ");
                if (_values[reviewIndex] != string.Empty)
                {
                    sbTemp.Append(_values[reviewIndex] + Environment.NewLine);
                }
                else
                {
                    sbTemp.Append("No review posted." + Environment.NewLine);
                }

                sbTemp.Append(Environment.NewLine + Environment.NewLine);
                sbTemp.Append("--------------- End of Book Record ---------------" + Environment.NewLine + Environment.NewLine + Environment.NewLine + Environment.NewLine);  

                

            }

            return sbTemp;
        }

        private void lstAuthors_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstAuthors.SelectedIndex == -1) { return; }


            string authorName = lstAuthors.SelectedItem.ToString();
            int authorInstances = 0;

            //MessageBox.Show(authorName);
            
            StringBuilder sbSingleAuthor = new StringBuilder();
            sbSingleAuthor.Clear();

            using (CsvFileReader reader = new CsvFileReader(FilePathAndName))
            {
                CsvRow row = new CsvRow();
                totalRows = getTotalRowsInCsvFile();
                int counter = 0;                
                while (reader.ReadRow(row) && counter <= totalRows)
                {
                    counter++;
                    string[] values = row.ToArray();

                    if (counter > 1)
                    {
                        if (values[authorIndex] == authorName)
                        {
                            sbSingleAuthor.Append(getAllRowsBySpecificAuthor(authorName, values).ToString());
                            authorInstances++;
                        }                        
                    }
                }
            }

            if (authorInstances > 0)
            {
                if(DialogResult.Yes == MessageBox.Show(authorName + " has " + authorInstances.ToString() + " book(s) that were found in the CSV file. Clear the textbox and show these books only?",
                    authorName, MessageBoxButtons.YesNo, MessageBoxIcon.Question)){
                        
                        textBox1.Clear();
                        textBox1.Text = sbSingleAuthor.ToString();
                        toolStripStatusLabel1.Text = String.Empty;
                        btnRefresh.Enabled = true;
                        btnRefresh.Visible = true;
                }
            }

        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            textBox1.Text = String.Empty;
            appendBookInfoToTextBox();
            btnRefresh.Visible = false;
            updateToolStripStatusBarLabelAfterLoadingFile();                
            lstAuthors.ClearSelected();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            fAbout.Show();
        }

        private void listView1_ItemActivate(object sender, EventArgs e)
        {            
            itemClicked = listView1.SelectedItems[0].Text;
            //MessageBox.Show(itemClicked);

            string msg = "Export row information for title \"" + itemClicked + "\"?";
            DialogResult dr = MessageBox.Show(msg,"Export?",MessageBoxButtons.YesNo, MessageBoxIcon.None);
            if(dr == DialogResult.Yes){                
                StringBuilder sb = new StringBuilder();
                sb = getEntireRowBySpecificTitle(itemClicked);
                MessageBox.Show(sb.ToString(), "To Be Exported...");
            }
        }
       


    }




    

}
